@@ -14,15 +14,17 @@ DESCRIPTION = """Usage is
 python3 -m skipole mynewproj /path/to/projectfiles
 
 Which creates a directory /path/to/projectfiles
-containing three sub directories:
+containing three sub directories and a python file:
 
-mynewproj - a subdirectory containing a new project
-skis - a project serving needed javascript files, necessary for all projects
-skiadmin - a project used to help develop 'mynewproj'
+mynewproj.py - a minimal python file which you willdevelop further.
+
+mynewproj - a subdirectory containing data and static files for your new project
+skis - a project subdirectory serving needed javascript files
+skiadmin - a project subdirectory used to help develop 'mynewproj'
 
 You should replace 'mynewproj' with your preferred name for a new project.
 This is an optional argument. If not given then only the skis and skiadmin
-projects will be created.
+projects directories will be created.
 
 The path "/path/to/projectfiles" must be given, and is the path to a directory
 where you will develop your project. Multiple projects can be created in one
@@ -93,8 +95,19 @@ template_newproj_directory = os.path.join(template_directory, 'newproj')
 if not os.path.isdir(template_newproj_directory):
     print("Error: Cannot find the template data for the newproj project")
     sys.exit(7)
+template_newproj_pyfile = os.path.join(template_directory, 'newproj.py')
+if not os.path.isfile(template_newproj_pyfile):
+    print("Error: Cannot find the template data for the newproj project")
+    sys.exit(8)
 
 
+if project_name is not None:
+    project_directory = os.path.join(projectfiles, project_name)
+    project_pyfile = os.path.join(projectfiles, project_name + ".py")
+
+# newproj python file
+newproj_pyfile = os.path.join(projectfiles, 'newproj.py')
+
 
 # If the given projectfiles directory does not exist
 if not os.path.isdir(projectfiles):
@@ -108,90 +121,59 @@ if not os.path.isdir(projectfiles):
     if project_name is None:
         # remove the new project
         shutil.rmtree(newproj_directory)
+        os.remove(newproj_pyfile)
         print("Project directory %s created" % (projectfiles,))
         sys.exit(0)
 
     # A project name is given, change the name of the newproj_directory to project_name
-    project_directory = os.path.join(projectfiles, project_name)
     os.rename(newproj_directory, project_directory)
+else:
+    # The given projectfiles does exist, so use it for the new project
+
+    # if skis exists, delete it, then copy the template
+    skis_directory = os.path.join(projectfiles, 'skis')
+    if os.path.isdir(skis_directory):
+        print("Replacing project skis")
+        shutil.rmtree(skis_directory)
+    shutil.copytree(template_skis_directory, skis_directory)
+
+    # if skiadmin exists, delete it, then copy the template
+    skiadmin_directory = os.path.join(projectfiles, 'skiadmin')
+    if os.path.isdir(skiadmin_directory):
+        print("Replacing project skiadmin")
+        shutil.rmtree(skiadmin_directory)
+    shutil.copytree(template_skiadmin_directory, skiadmin_directory)
+
+    if project_name is None:
+        sys.exit(0)
+
+    if os.path.isdir(project_directory) or os.path.isfile(project_pyfile):
+        print("skis and skiadmin have been replaced, however project %s already exists, and has not been altered." % (project_name,))
+        sys.exit(0)
+
+    # create a new project
+    shutil.copytree(template_newproj_directory, project_directory)
+    shutil.copyfile(template_newproj_pyfile, newproj_pyfile)
 
 
-    new_code_path = os.path.join(project_directory, "code", project_name + ".py")
-    print("Creating %s" % (new_code_path,))
-
-    # change newproj.py to project_name.py, including the contents
-    ## READ newproj.py
-    newproj_file = os.path.join(project_directory, "code", "newproj.py")
-    with open(newproj_file, "r") as tnf:
-        templatecontents = tnf.read()
-    os.remove(newproj_file)
-    ## REPLACE newproj with the new project name
-    newcontents = templatecontents.replace("newproj", project_name)
-    ## WRITE the new file
-    with open(new_code_path, "w") as pf:
-        pf.write(newcontents)
-
-    # change data/project.json contents
-    ## READ project.json
-    data_file = os.path.join(project_directory, "data", "project.json")
-    with open(data_file, "r") as tdf:
-        templatedatacontents = tdf.read()
-    os.remove(data_file)
-    ## REPLACE newproj with the new project name
-    newdatacontents = templatedatacontents.replace("newproj", project_name)
-    ## WRITE the new file
-    with open(data_file, "w") as tdf2:
-        tdf2.write(newdatacontents)
+# new_code_path -> project_pyfile
+# newproj_file -> newproj_pyfile
 
 
-    print("Project directory %s created, with new project %s" % (projectfiles,project_name))
-    print("""Use the command:
-python3 %s
-To serve the project at url localhost:8000
-and localhost:8000/skiadmin for the admin pages""" % (new_code_path,))
-    sys.exit(0)
-
-# The given projectfiles does exist, so use it for the new project
-
-print("Replacing project skis")
-# if skis exists, delete it, then copy the template
-skis_directory = os.path.join(projectfiles, 'skis')
-if os.path.isdir(skis_directory):
-    shutil.rmtree(skis_directory)
-shutil.copytree(template_skis_directory, skis_directory)
-
-print("Replacing project skiadmin")
-# if skiadmin exists, delete it, then copy the template
-skiadmin_directory = os.path.join(projectfiles, 'skiadmin')
-if os.path.isdir(skiadmin_directory):
-    shutil.rmtree(skiadmin_directory)
-shutil.copytree(template_skiadmin_directory, skiadmin_directory)
-
-if project_name is None:
-    print("skis and skiadmin have been replaced.")
-    sys.exit(0)
-
-project_directory = os.path.join(projectfiles, project_name)
-if os.path.isdir(project_directory):
-    print("skis and skiadmin have been replaced, however project % already exists, and has not been altered." % (project_name,))
-    sys.exit(0)
-
-# create a new project
-shutil.copytree(template_newproj_directory, project_directory)
-
-new_code_path = os.path.join(project_directory, "code", project_name + ".py")
-print("Creating %s" % (new_code_path,))
+print("Creating %s" % (project_pyfile,))
 
 # change newproj.py to project_name.py, including the contents
 ## READ newproj.py
-newproj_file = os.path.join(project_directory, "code", "newproj.py")
-with open(newproj_file, "r") as tnf:
+
+with open(newproj_pyfile, "r") as tnf:
     templatecontents = tnf.read()
-os.remove(newproj_file)
+os.remove(newproj_pyfile)
 ## REPLACE newproj with the new project name
 newcontents = templatecontents.replace("newproj", project_name)
+## REPLACE the 'os.path.dirname(os.path.realpath(__file__))' call with the new projectfiles
+newcontents = newcontents.replace("os.path.dirname(os.path.realpath(__file__))", "\"" + projectfiles + "\"")
 ## WRITE the new file
-with open(new_code_path, "w") as pf:
+with open(project_pyfile, "w") as pf:
     pf.write(newcontents)
 
 # change data/project.json contents
@@ -206,12 +188,10 @@ newdatacontents = templatedatacontents.replace("newproj", project_name)
 with open(data_file, "w") as tdf2:
     tdf2.write(newdatacontents)
 
-print("skis and skiadmin have been replaced, and new project %s created." % (project_name,))
 print("""Use the command:
 python3 %s
 To serve the project at url localhost:8000
-and localhost:8000/skiadmin for the admin pages""" % (new_code_path,))
+and localhost:8000/skiadmin for the admin pages""" % (project_pyfile,))
 sys.exit(0)
 
 
-
