@@ -47,9 +47,10 @@ under key 'received_data' which contains a dictionary of widgfield tuples:values
     # Options for the fields argument
     field_options = {'fields': True,                  # If False, no fields are expected
                      'widgfields':True,               # If True, fields are widgfields, if False, can be other constants
-                     'fields_optional': True,        # if fields is True, then False here means fields must be supplied
+                     'fields_optional': True,         # if fields is True, then False here means fields must be supplied
                      'widgfield_values':False,        # If True the field values are widgfields
-                     'field_values': False,            # if True, field values are used
+                     'field_values': False,           # if True, field values are used
+                     'field_keys': False,             # if field_values is True, and this field_keys is True, the values supplied are dictionary keys
                      'empty_values_allowed':True,     # If True, '' is a valid value, if False, some data must be provided
                      'single_field': False}           # Multiple fields accepted
 
@@ -58,7 +59,7 @@ under key 'received_data' which contains a dictionary of widgfield tuples:values
         "Gets the target page, filling in the form data"
         if caller_page is None:
             if self.alternate_ident:
-                return self.get_alternate_page(skicall, {}, None, ident_list, proj_ident, rawformdata)
+                return self.get_alternate_page(proj_ident)
             else:
                 if skiboot.get_debug():
                     responder_ident = ident_list[-1]
@@ -66,7 +67,7 @@ under key 'received_data' which contains a dictionary of widgfield tuples:values
                     raise ValidateError(message)
                 else:
                     raise ValidateError(message='No caller page, and no alternate ident set in AllowedFields responder')
-        self._check_allowed_callers(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
+        self._check_allowed_callers(caller_page, ident_list, proj_ident)
         # previous caller is allowed, now check the form data
 
         if not self.fields:
@@ -85,8 +86,8 @@ under key 'received_data' which contains a dictionary of widgfield tuples:values
                         self._submit_data(ident_list, skicall)
                     except FailPage as e:
                         # raises a PageError exception
-                        self.raise_error_page([e.errormessage], skicall, {}, caller_page, ident_list, proj_ident, rawformdata)
-                return self.get_target_page(skicall, {}, caller_page, ident_list, proj_ident, rawformdata)
+                        self.raise_error_page(proj_ident, [e.errormessage], e.failpage)
+                return self.get_target_page(proj_ident)
 
         # generate new form data
         new_form_data = {}
@@ -129,10 +130,10 @@ under key 'received_data' which contains a dictionary of widgfield tuples:values
                 self._submit_data(ident_list, skicall)
             except FailPage as e:
                 # raises a PageError exception
-                self.raise_error_page([e.errormessage], skicall, validated_form_data, caller_page, ident_list, proj_ident, rawformdata)
+                self.raise_error_page(proj_ident, [e.errormessage], e.failpage)
                 
         # return the target page
-        return self.get_target_page(skicall, validated_form_data, caller_page, ident_list, proj_ident, rawformdata)
+        return self.get_target_page(proj_ident)
 
 
 class StoreData(Respond):
@@ -154,6 +155,7 @@ class StoreData(Respond):
                      'widgfield_values':False,        # If True the field values are widgfields
                      'fields_optional': False,        # if fields is True, then False here means fields must be supplied
                      'field_values': False,           # if True, field values are used
+                     'field_keys': False,             # if field_values is True, and this field_keys is True, the values supplied are dictionary keys
                      'empty_values_allowed':True,     # If True, '' is a valid value, if False, some data must be provided
                      'single_field': False}           # Multiple fields accepted
 
@@ -163,7 +165,7 @@ class StoreData(Respond):
         if self.validate_option:
             if caller_page is None:
                 # raises a PageError exception
-                self.raise_error_page([], skicall, validated_form_data, caller_page, ident_list, proj_ident, rawformdata)
+                self.raise_error_page(proj_ident, [], None)
             validated_form_data = self._validate_fields(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
         else:
             validated_form_data = form_data
@@ -188,9 +190,9 @@ class StoreData(Respond):
                 self._submit_data(ident_list, skicall)
             except FailPage as e:
                 # raises a PageError exception
-                self.raise_error_page([e.errormessage], skicall, validated_form_data, caller_page, ident_list, proj_ident, rawformdata)
+                self.raise_error_page(proj_ident, [e.errormessage], e.failpage)
  
-        return self.get_target_page(skicall, validated_form_data, caller_page, ident_list, proj_ident, rawformdata)
+        return self.get_target_page(proj_ident)
 
 
 class StoreDataKeyed(Respond):
@@ -211,6 +213,7 @@ class StoreDataKeyed(Respond):
                      'widgfield_values':False,        # If True the field values are widgfields
                      'fields_optional': False,        # if fields is True, then False here means fields must be supplied
                      'field_values': True,            # if True, field values are used
+                     'field_keys': True,              # if field_values is True, and this field_keys is True, the values supplied are dictionary keys
                      'empty_values_allowed':False,    # If True, '' is a valid value, if False, some data must be provided
                      'single_field': False}           # Multiple fields accepted
 
@@ -220,7 +223,7 @@ class StoreDataKeyed(Respond):
         if self.validate_option:
             if caller_page is None:
                 # raises a PageError exception
-                self.raise_error_page([], skicall, validated_form_data, caller_page, ident_list, proj_ident, rawformdata)
+                self.raise_error_page(proj_ident, [], None)
             validated_form_data = self._validate_fields(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
         else:
             validated_form_data = form_data
@@ -245,9 +248,9 @@ class StoreDataKeyed(Respond):
                 self._submit_data(ident_list, skicall)
             except FailPage as e:
                 # raises a PageError exception
-                self.raise_error_page([e.errormessage], skicall, validated_form_data, caller_page, ident_list, proj_ident, rawformdata)
+                self.raise_error_page(proj_ident, [e.errormessage], e.failpage)
 
-        return self.get_target_page(skicall, validated_form_data, caller_page, ident_list, proj_ident, rawformdata)
+        return self.get_target_page(proj_ident)
 
 
 class AllowStoreKeyed(Respond):
@@ -285,6 +288,7 @@ with keys equal to the field values set here.
                      'fields_optional': False,        # if fields is True, then False here means fields must be supplied
                      'widgfield_values':False,        # If True the field values are widgfields
                      'field_values': True,            # if True, field values are used
+                     'field_keys': True,              # if field_values is True, and this field_keys is True, the values supplied are dictionary keys
                      'empty_values_allowed':False,    # If True, '' is a valid value, if False, some data must be provided
                      'single_field': False}           # Multiple fields accepted
 
@@ -294,10 +298,10 @@ with keys equal to the field values set here.
 
         if caller_page is None:
             if self.alternate_ident:
-                return self.get_alternate_page(skicall, {}, None, ident_list, proj_ident, rawformdata)
+                return self.get_alternate_page(proj_ident)
             else:
                 raise ValidateError(message='No caller page, and no alternate ident set in AllowStoreKeyed responder')
-        self._check_allowed_callers(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
+        self._check_allowed_callers(caller_page, ident_list, proj_ident)
         # previous caller is allowed
         
         if not self.fields:
@@ -312,8 +316,8 @@ with keys equal to the field values set here.
                         self._submit_data(ident_list, skicall)
                     except FailPage as e:
                         # raises a PageError exception
-                        self.raise_error_page([e.errormessage], skicall, {}, caller_page, ident_list, proj_ident, rawformdata)
-                return self.get_target_page(skicall, {}, caller_page, ident_list, proj_ident, rawformdata)
+                        self.raise_error_page(proj_ident, [e.errormessage], e.failpage)
+                return self.get_target_page(proj_ident)
         
         # check fields received are specified in this responder
         for field in form_data:
@@ -345,8 +349,8 @@ with keys equal to the field values set here.
                 self._submit_data(ident_list, skicall)
             except FailPage as e:
                 # raises a PageError exception
-                self.raise_error_page([e.errormessage], skicall, validated_form_data, caller_page, ident_list, proj_ident, rawformdata)
-        return self.get_target_page(skicall, validated_form_data, caller_page, ident_list, proj_ident, rawformdata)
+                self.raise_error_page(proj_ident, [e.errormessage], e.failpage)
+        return self.get_target_page(proj_ident)
         
         
 class AllowStore(Respond):
@@ -373,6 +377,7 @@ class AllowStore(Respond):
                      'widgfield_values':False,        # If True the field values are widgfields
                      'fields_optional': False,        # if fields is True, then False here means fields must be supplied
                      'field_values': False,           # if True, field values are used
+                     'field_keys': False,             # if field_values is True, and this field_keys is True, the values supplied are dictionary keys
                      'empty_values_allowed':True,     # If True, '' is a valid value, if False, some data must be provided
                      'single_field': False}           # Multiple fields accepted
 
@@ -382,7 +387,7 @@ class AllowStore(Respond):
    
         if caller_page is None:
             if self.alternate_ident:
-                return self.get_alternate_page(skicall, {}, None, ident_list, proj_ident, rawformdata)
+                return self.get_alternate_page(proj_ident)
             else:
                 if skiboot.get_debug():
                     responder_ident = ident_list[-1]
@@ -390,7 +395,7 @@ class AllowStore(Respond):
                     raise ValidateError(message)
                 else:
                     raise ValidateError(message='No caller page, and no alternate ident set in AllowStore responder')
-        self._check_allowed_callers(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
+        self._check_allowed_callers(caller_page, ident_list, proj_ident)
         # previous caller is allowed
         
         if not self.fields:
@@ -410,8 +415,8 @@ class AllowStore(Respond):
                         self._submit_data(ident_list, skicall)
                     except FailPage as e:
                         # raises a PageError exception
-                        self.raise_error_page([e.errormessage], skicall, {}, caller_page, ident_list, proj_ident, rawformdata)
-                return self.get_target_page(skicall, {}, caller_page, ident_list, proj_ident, rawformdata)
+                        self.raise_error_page(proj_ident, [e.errormessage], e.failpage)
+                return self.get_target_page(proj_ident)
         
         # check fields received are specified in this responder
         for field in form_data:
@@ -447,9 +452,9 @@ class AllowStore(Respond):
                 self._submit_data(ident_list, skicall)
             except FailPage as e:
                 # raises a PageError exception
-                self.raise_error_page([e.errormessage], skicall, validated_form_data, caller_page, ident_list, proj_ident, rawformdata)
+                self.raise_error_page(proj_ident, [e.errormessage], e.failpage)
  
-        return self.get_target_page(skicall, validated_form_data, caller_page, ident_list, proj_ident, rawformdata)
+        return self.get_target_page(proj_ident)
 
 
 class PrettyFormData(Respond):
@@ -470,6 +475,7 @@ class PrettyFormData(Respond):
                      'widgfield_values':False,        # If True the field values are widgfields
                      'fields_optional': True,        # if fields is True, then False here means fields must be supplied
                      'field_values': False,            # if True, field values are used
+                     'field_keys': False,             # if field_values is True, and this field_keys is True, the values supplied are dictionary keys
                      'empty_values_allowed':True,     # If True, '' is a valid value, if False, some data must be provided
                      'single_field': False}           # Multiple fields accepted
 
@@ -501,9 +507,9 @@ class PrettyFormData(Respond):
             self._submit_data(ident_list, skicall)
         except FailPage as e:
             # raises a PageError exception
-            self.raise_error_page([e.errormessage], skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
+            self.raise_error_page(proj_ident, [e.errormessage], e.failpage)
         # so all ok, get the target page
-        return self.get_target_page(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
+        return self.get_target_page(proj_ident)
 
 
 
@@ -524,6 +530,7 @@ class Accept(Respond):
                      'widgfield_values':False,        # If True the field values are widgfields
                      'fields_optional': True,        # if fields is True, then False here means fields must be supplied
                      'field_values': False,            # if True, field values are used
+                     'field_keys': False,             # if field_values is True, and this field_keys is True, the values supplied are dictionary keys
                      'empty_values_allowed':True,     # If True, '' is a valid value, if False, some data must be provided
                      'single_field': False}           # Multiple fields accepted
 
@@ -548,9 +555,9 @@ class Accept(Respond):
             self._submit_data(ident_list, skicall)
         except FailPage as e:
             # raises a PageError exception
-            self.raise_error_page([e.errormessage], skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
+            self.raise_error_page(proj_ident, [e.errormessage], e.failpage)
         
-        return self.get_target_page(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
+        return self.get_target_page(proj_ident)
 
 
 
@@ -578,6 +585,7 @@ class AllowedAccept(Respond):
                      'widgfield_values':False,        # If True the field values are widgfields
                      'fields_optional': True,        # if fields is True, then False here means fields must be supplied
                      'field_values': False,            # if True, field values are used
+                     'field_keys': False,             # if field_values is True, and this field_keys is True, the values supplied are dictionary keys
                      'empty_values_allowed':True,     # If True, '' is a valid value, if False, some data must be provided
                      'single_field': False}           # Multiple fields accepted
 
@@ -586,7 +594,7 @@ class AllowedAccept(Respond):
 
         if caller_page is None:
             if self.alternate_ident:
-                return self.get_alternate_page(skicall, {}, None, ident_list, proj_ident, rawformdata)
+                return self.get_alternate_page(proj_ident)
             else:
                 if skiboot.get_debug():
                     responder_ident = ident_list[-1]
@@ -594,7 +602,7 @@ class AllowedAccept(Respond):
                     raise ValidateError(message)
                 else:
                     raise ValidateError(message='No caller page, and no alternate ident set in AllowedAccept responder')
-        self._check_allowed_callers(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
+        self._check_allowed_callers(caller_page, ident_list, proj_ident)
         # previous caller is allowed, now store the received form data
 
 
@@ -609,9 +617,9 @@ class AllowedAccept(Respond):
             self._submit_data(ident_list, skicall)
         except FailPage as e:
             # raises a PageError exception
-            self.raise_error_page([e.errormessage], skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
+            self.raise_error_page(proj_ident, [e.errormessage], e.failpage)
         
-        return self.get_target_page(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
+        return self.get_target_page(proj_ident)
 
 
 
@@ -627,6 +635,7 @@ class PageData(Respond):
                      'widgfield_values':False,        # If True the field values are widgfields
                      'fields_optional': False,        # if fields is True, then False here means fields must be supplied
                      'field_values': True,            # if True, field values are used
+                     'field_keys': False,             # if field_values is True, and this field_keys is True, the values supplied are dictionary keys
                      'empty_values_allowed':True,     # If True, '' is a valid value, if False, some data must be provided
                      'single_field': False}           # Multiple fields accepted
 
@@ -641,5 +650,5 @@ class PageData(Respond):
             else:
                 # widgfield is actually a string, such as 'show_error'
                 skicall.page_data[str_field] = value
-        return self.get_target_page(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
+        return self.get_target_page(proj_ident)
 
