@@ -22,10 +22,10 @@ class Pin4(Widget):
                         'hide':FieldArg("boolean", False, jsonset=True),
                         'autofocus':FieldArg("boolean", True),
                         'action':FieldArg("url", ''),
-                        'hidden_field1':FieldArg("text", '', valdt=True),
-                        'hidden_field2':FieldArg("text", '', valdt=True),
-                        'hidden_field3':FieldArg("text", '', valdt=True),
-                        'hidden_field4':FieldArg("text", '', valdt=True),
+                        'hidden_field1':FieldArg("text", '', valdt=True, jsonset=True),
+                        'hidden_field2':FieldArg("text", '', valdt=True, jsonset=True),
+                        'hidden_field3':FieldArg("text", '', valdt=True, jsonset=True),
+                        'hidden_field4':FieldArg("text", '', valdt=True, jsonset=True),
                         'button_text':FieldArg("text", 'Submit'),
                         'pin1':FieldArg("boolean", True, valdt=True),
                         'pin2':FieldArg("boolean", True, valdt=True),
@@ -264,10 +264,10 @@ class NamePasswd1(Widget):
                         'action_json':FieldArg("url", ''),
                         'error_class':FieldArg("cssclass", ""),
                         'inputdiv_class':FieldArg("cssclass", ''),
-                        'hidden_field1':FieldArg("text", '', valdt=True),
-                        'hidden_field2':FieldArg("text", '', valdt=True),
-                        'hidden_field3':FieldArg("text", '', valdt=True),
-                        'hidden_field4':FieldArg("text", '', valdt=True),
+                        'hidden_field1':FieldArg("text", '', valdt=True, jsonset=True),
+                        'hidden_field2':FieldArg("text", '', valdt=True, jsonset=True),
+                        'hidden_field3':FieldArg("text", '', valdt=True, jsonset=True),
+                        'hidden_field4':FieldArg("text", '', valdt=True, jsonset=True),
                         'div1_class':FieldArg("cssclass", ''),
                         'label1':FieldArg("text", 'Username:'),
                         'label_class1':FieldArg("cssclass", ''),
