@@ -111,9 +111,11 @@ SKIPOLE.checkbox.CheckBoxTable1.prototype.setvalues = function (fieldlist, resul
     var row_classes = this.fieldarg_in_result('row_classes', result, fieldlist);
     var itemschecked = this.fieldarg_in_result('checked', result, fieldlist);
     var self = this;
-    var nameschecked = itemschecked.map( function(item) {
-        return self.formname('checked') + "-" + item;
-        })
+    if (itemschecked && itemschecked.length) {
+        var nameschecked = itemschecked.map( function(item) {
+            return self.formname('checked') + "-" + item;
+            })
+        }
     var index = 0;
     var header = false;
     if (the_widg.find('th').length) {
@@ -127,13 +129,21 @@ SKIPOLE.checkbox.CheckBoxTable1.prototype.setvalues = function (fieldlist, resul
         else {
             // for each row
             // set its class
-            $(this).attr("class", row_classes[index]);
+            if (row_classes && row_classes.length) {
+                if (row_classes[index] !== null) {
+                    $(this).attr("class", row_classes[index]);
+                    }
+                }
             var cells = $(this).children();
-            if (col1) {
-                $(cells[0]).text(col1[index]);
+            if (col1 && col1.length) {
+                if (col1[index] !== null) {
+                    $(cells[0]).text(col1[index]);
+                    }
                  }
-            if (col2) {
-                $(cells[1]).text(col2[index]);
+            if (col2 && col2.length) {
+                if (col2[index] !== null) {
+                    $(cells[1]).text(col2[index]);
+                    }
                  }
             if (nameschecked) {
                 // Remove checked status from all rows
